@@ -76,6 +76,7 @@ def _parse_pabot_args(args):  # type: (List[str]) -> Tuple[List[str], Dict[str,
         "artifactsinsubfolders": False,
         "shardindex": 0,
         "shardcount": 1,
+        "chunk": False,
     }
     argumentfiles = []
     while args and (
@@ -98,6 +99,7 @@ def _parse_pabot_args(args):  # type: (List[str]) -> Tuple[List[str], Dict[str,
                 "artifactsinsubfolders",
                 "help",
                 "shard",
+                "chunk",
             ]
         ]
         or ARGSMATCHER.match(args[0])
@@ -119,6 +121,10 @@ def _parse_pabot_args(args):  # type: (List[str]) -> Tuple[List[str], Dict[str,
             pabot_args["verbose"] = True
             args = args[1:]
             continue
+        if args[0] == "--chunk":
+            pabot_args["chunk"] = True
+            args = args[1:]
+            continue
         if args[0] == "--resourcefile":
             pabot_args["resourcefile"] = args[1]
             args = args[2:]
