@@ -16,7 +16,7 @@
 #
 #  partly based on work by Nokia Solutions and Networks Oyj
 """A parallel executor for Robot Framework test cases.
-Version 2.4.0
+Version 2.5.5
 
 Supports all Robot Framework command line options and also following
 options (these must be before normal RF options):
@@ -64,10 +64,13 @@ options (these must be before normal RF options):
   For example "--argumentfile1 arg1.txt --argumentfile2 arg2.txt".
 
 --shard [SHARD]/[SHARD COUNT]
-  Optionally split execution into smaller pieces. This is
-  typically used for distributing testing to multiple machines.
+  Optionally split execution into smaller pieces. This can
+  be used for distributing testing to multiple machines.
 
-Copyright 2019 Mikko Korpela - Apache 2 License
+--chunk
+  Optionally chunk tests to PROCESSES number of robot runs.
+
+Copyright 2022 Mikko Korpela - Apache 2 License
 """
 
 from __future__ import absolute_import, print_function
@@ -296,8 +299,6 @@ def _try_execute_and_wait(
     if plib:
         _increase_completed(plib, my_index)
         is_ignored = _is_ignored(plib, caller_id)
-        if is_ignored and os.path.isdir(outs_dir):
-            shutil.rmtree(outs_dir)
     # Thread-safe list append
     _ALL_ELAPSED.append(elapsed)
     _result_to_stdout(
@@ -313,7 +314,8 @@ def _try_execute_and_wait(
         verbose,
         show_stdout_on_failure,
     )
-
+    if is_ignored and os.path.isdir(outs_dir):
+        shutil.rmtree(outs_dir)
 
 def _result_to_stdout(
     elapsed,
@@ -791,8 +793,15 @@ def shard_suites(suite_names, pabot_args):
     shard_index = pabot_args["shardindex"]
     shard_count = pabot_args["shardcount"]
     if shard_index > shard_count:
-        return []
-    q, r = divmod(len(suite_names), shard_count)
+        raise DataError(
+            f"Shard index ({shard_index}) greater than shard count ({shard_count})."
+        )
+    items_count = len(suite_names)
+    if items_count < shard_count:
+        raise DataError(
+            f"Not enought items ({items_count}) for shard cound ({shard_count})."
+        )
+    q, r = divmod(items_count, shard_count)
     return suite_names[
         (shard_index - 1) * q
         + min(shard_index - 1, r) : shard_index * q
@@ -1194,14 +1203,31 @@ def _options_for_rebot(options, start_time_string, end_time_string):
     if ROBOT_VERSION >= "2.8":
         options["monitormarkers"] = "off"
     for key in [
+        "console",
+        "consolemarkers",
+        "consolewidth",
+        "debugfile",
+        "dotted",
+        "dryrun",
+        "exitonerror",
+        "exitonfailure",
+        "extension",
+        "listener",
+        "loglevel",
+        "language",
+        "maxassignlength",
+        "maxerrorlines",
+        "monitorcolors",
+        "prerunmodifier",
+        "quiet",
+        "randomize",
+        "runemptysuite",
+        "rerunfailed",
         "skip",
         "skiponfailure",
+        "skipteardownonexit",
         "variable",
         "variablefile",
-        "listener",
-        "prerunmodifier",
-        "monitorcolors",
-        "randomize",
     ]:
         if key in rebot_options:
             del rebot_options[key]
@@ -1246,18 +1272,34 @@ def keyboard_interrupt(*args):
     CTRL_C_PRESSED = True
 
 
-def _parallel_execute(items, processes):
+def _parallel_execute(
+    items, processes, datasources, outs_dir, opts_for_run, pabot_args
+):
     original_signal_handler = signal.signal(signal.SIGINT, keyboard_interrupt)
     pool = ThreadPool(processes)
-    result = pool.map_async(execute_and_wait_with, items, 1)
-    pool.close()
-    while not result.ready():
+    results = [pool.map_async(execute_and_wait_with, items, 1)]
+    delayed_result_append = 0
+    new_items = []
+    while not all(result.ready() for result in results) or delayed_result_append > 0:
         # keyboard interrupt is executed in main thread
         # and needs this loop to get time to get executed
         try:
             time.sleep(0.1)
         except IOError:
             keyboard_interrupt()
+        dynamic_items = _get_dynamically_created_execution_items(
+            datasources, outs_dir, opts_for_run, pabot_args
+        )
+        if dynamic_items:
+            new_items += dynamic_items
+            # Because of last level construction, wait for more.
+            delayed_result_append = 3
+        delayed_result_append = max(0, delayed_result_append - 1)
+        if new_items and delayed_result_append == 0:
+            _construct_last_levels([new_items])
+            results.append(pool.map_async(execute_and_wait_with, new_items, 1))
+            new_items = []
+    pool.close()
     signal.signal(signal.SIGINT, original_signal_handler)
 
 
@@ -1721,17 +1763,19 @@ def _construct_last_levels(all_items):
                     names.append(suite.name)
             else:
                 names.append(item.execution_item.name)
+    index = 0
     for items in all_items:
         for item in items:
             if isinstance(item.execution_item, SuiteItems):
                 for suite in item.execution_item.suites:
                     item.last_level = _find_ending_level(
-                        suite.name, names[item.index + 1 :]
+                        suite.name, names[index + 1:]
                     )
             else:
                 item.last_level = _find_ending_level(
-                    item.execution_item.name, names[item.index + 1 :]
+                    item.execution_item.name, names[index + 1:]
                 )
+            index += 1
 
 
 def _initialize_queue_index():
@@ -1752,16 +1796,16 @@ def _initialize_queue_index():
     raise RuntimeError("Can not connect to PabotLib at %s" % _PABOTLIBURI)
 
 
-def _add_dynamically_created_execution_items(
-    execution_items, datasources, outs_dir, opts_for_run, pabot_args
+def _get_dynamically_created_execution_items(
+    datasources, outs_dir, opts_for_run, pabot_args
 ):
     global _COMPLETED_LOCK, _NOT_COMPLETED_INDEXES, _NUMBER_OF_ITEMS_TO_BE_EXECUTED
     if not _pabotlib_in_use():
-        return
+        return None
     plib = Remote(_PABOTLIBURI)
     new_suites = plib.run_keyword("get_added_suites", [], {})
     if len(new_suites) == 0:
-        return
+        return None
     suite_group = [DynamicSuiteItem(s, v) for s, v in new_suites]
     items = [
         QueueItem(
@@ -1781,7 +1825,7 @@ def _add_dynamically_created_execution_items(
         _NUMBER_OF_ITEMS_TO_BE_EXECUTED += len(items)
         for item in items:
             _NOT_COMPLETED_INDEXES.append(item.index)
-    execution_items.insert(0, items)
+    return items
 
 
 def main(args=None):
@@ -1819,13 +1863,10 @@ def main(args=None):
         ordering = pabot_args.get("ordering")
         if ordering:
             suite_names = _preserve_order(suite_names, ordering)
-        if pabot_args["processes"] > 1:
-            suite_names = _group_by_wait(_group_by_groups(suite_names))
+        if pabot_args["chunk"]:
+            suite_names = _chunked_suite_names(suite_names, pabot_args["processes"])
         else:
-            all_group = GroupItem()
-            for s in suite_names:
-                all_group.add(s)
-            suite_names = [[all_group]]
+            suite_names = _group_by_wait(_group_by_groups(suite_names))
         if not suite_names or suite_names == [[]]:
             _write("No tests to execute")
             if not options.get("runemptysuite", False):
@@ -1835,9 +1876,13 @@ def main(args=None):
         )
         while execution_items:
             items = execution_items.pop(0)
-            _parallel_execute(items, pabot_args["processes"])
-            _add_dynamically_created_execution_items(
-                execution_items, datasources, outs_dir, opts_for_run, pabot_args
+            _parallel_execute(
+                items,
+                pabot_args["processes"],
+                datasources,
+                outs_dir,
+                opts_for_run,
+                pabot_args,
             )
         result_code = _report_results(
             outs_dir,
@@ -1867,5 +1912,21 @@ def main(args=None):
         _stop_message_writer()
 
 
+def _chunked_suite_names(suite_names, processes):
+    q, r = divmod(len(suite_names), processes)
+    result = []
+    for index in range(processes):
+        chunk = suite_names[
+            (index) * q + min(index, r) : (index + 1) * q + min((index + 1), r)
+        ]
+        if len(chunk) == 0:
+            continue
+        grouped = GroupItem()
+        for item in chunk:
+            grouped.add(item)
+        result.append(grouped)
+    return [result]
+
+
 if __name__ == "__main__":
     main()
